{% extends 'base.html' %}
{% load crispy_forms_tags %}

{% block head %}
<title>Delete Form</title>
{% endblock %}

{% block content %}

<nav aria-label="breadcrumb">
  <ol class="breadcrumb">
    <li class="breadcrumb-item">
      <a href="{% url 'form_creator:form_list' %}">Forms</a>
    </li>
    <li class="breadcrumb-item">
      <a href="{{ object.get_absolute_url }}">{{ object|truncatechars:20 }}</a>
    </li>
    <li class="breadcrumb-item active" aria-current="page">Delete</li>
  </ol>
</nav>

<h1>Delete Form</h1>
<p>Are you sure you want to delete this form: {{ object }}.</p>
<p>
  Deleting this question will delete all questions and answers associated with
  it.
</p>

<form method="POST">
  {% csrf_token %}
  <input type="submit" class="btn btn-primary" value="Confirm">
  <a href="{{ object.get_absolute_url }}" class="btn btn-danger">Cancel</a>
</form>

{% endblock %}